@@ -37,7 +37,7 @@
 #include <gdaemonfilemonitor.h>
 #include <gdaemonfileenumerator.h>
 #include <glib/gi18n-lib.h>
-#include "gdbusutils.h"
+#include "gvfsdbusutils.h"
 #include "gmountoperationdbus.h"
 #include <gio/gio.h>
 #include "metatree.h"
@@ -1684,7 +1684,7 @@ g_daemon_file_poll_mountable (GFile               *file,
 		      callback, user_data,
 		      poll_mountable_async_cb,
 		      NULL,
-		      0,
+		      NULL,
                       0);
 }
 
