@@ -30,7 +30,7 @@
 #include <stdlib.h>
 #include <glib/gprintf.h>
 
-#include "gdbusutils.h"
+#include "gvfsdbusutils.h"
 #include "gvfsproxyvolumemonitordaemon.h"
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -44,7 +44,6 @@ static GList *outstanding_mount_op_objects = NULL;
 static GHashTable *unique_names_being_watched = NULL;
 
 /* #define DEBUG_ENABLED */
-#define DEBUG_ENABLED
 
 #ifdef DEBUG_ENABLED
 static void
@@ -599,6 +598,8 @@ append_volume (GVolume *volume, DBusMessageIter *iter_array)
       DBusMessageIter iter_dict_entry;
       char *id_value;
       id_value = g_volume_get_identifier (volume, identifiers[n]);
+      if (id_value == NULL)
+        continue;
       dbus_message_iter_open_container (&iter_identifiers,
                                         DBUS_TYPE_DICT_ENTRY,
                                         NULL,
@@ -916,12 +917,11 @@ handle_mount_op_reply (DBusConnection *connection, DBusMessage *message)
   dbus_bool_t anonymous;
   DBusError dbus_error;
   DBusHandlerResult ret;
-  GList *volumes, *l;
+  GList *l;
   DBusMessage *reply;
   GMountOperation *mount_operation;
   const gchar *sender;
 
-  volumes = NULL;
   decoded_password = NULL;
   ret = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
@@ -950,6 +950,8 @@ handle_mount_op_reply (DBusConnection *connection, DBusMessage *message)
   sender = dbus_message_get_sender (message);
 
   /* Find the op */
+  mount_operation = NULL;
+
   for (l = outstanding_mount_op_objects; l != NULL; l = l->next)
     {
       GMountOperation *op = G_MOUNT_OPERATION (l->data);
@@ -1805,10 +1807,8 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *user_da
             {
               GMountOperation *op = G_MOUNT_OPERATION (l->data);
               const gchar *owner;
-              const gchar *id;
 
               owner = g_object_get_data (G_OBJECT (op), "mount_op_owner");
-              id = g_object_get_data (G_OBJECT (op), "mount_op_id");
               if (g_strcmp0 (owner, name) == 0)
                 {
                   print_debug ("****** name `%s' has an outstanding mount operation object, aborting it",
@@ -2063,7 +2063,7 @@ g_vfs_proxy_volume_monitor_daemon_main (int argc,
                                         const char *dbus_name,
                                         GType volume_monitor_type)
 {
-  int rc;
+  int rc G_GNUC_UNUSED;
   int ret;
   GMainLoop *loop;
   DBusError dbus_error;
